local math = math
local pi = math.pi

function updateDoors(dt)
    for i = 1, #mapDoor do
        if mapDoor[i].opening then
            mapDoor[i].slide = mapDoor[i].slide - dt * mapDoor[i].speed
            if mapDoor[i].slide < 0 then
                mapDoor[i].slide = 0
                if mapDoor[i].timer == 0 then mapDoor[i].timer = 2 end
            end
        elseif mapDoor[i].opening == false then
            mapDoor[i].slide = mapDoor[i].slide + dt * mapDoor[i].speed
            if mapDoor[i].slide > 1 then
                mapDoor[i].slide = 1
            end
        end
        if mapDoor[i].slide == 0 then
            mapDoor[i].opened = true
        else
            mapDoor[i].opened = false
        end
        if mapDoor[i].timer > 0 and mapDoor[i].slide == 0 then
            --print(math.ceil(mapDoor[i].timer), mapDoor[i].x, mapDoor[i].y, player.x, player.y)
--[[            local enemyInWay = false
        	for i, e in pairs(mapEnemy) do
        	    print(i, e.id)
        	    if (math.floor(e.x) == mapDoor[i].x-1 and math.floor(e.y) == mapDoor[i].y-1) then
            	    enemyInWay = true
            	end
        	end]]

            if (math.floor(player.x) == mapDoor[i].x-1 and math.floor(player.y) == mapDoor[i].y-1) or enemyInWay then
                mapDoor[i].timer = .1
            else
                mapDoor[i].timer = mapDoor[i].timer - dt
            end
            if mapDoor[i].timer < 0 then
                mapDoor[i].opening = false
                mapDoor[i].timer = 2
            end
        end
    end
end

function loadDoors()
	for i = 1, #mapDoor do
	    mapDoor[i].opening = false
	    mapDoor[i].opened = false
	    mapDoor[i].slide = 1
	    mapDoor[i].speed = mapDoor[i].speed or 2
	    mapDoor[i].timer = 0
	    mapDoor[i].texture = mapDoor[i].texture or 13
	    mapDoor[i].frame = mapDoor[i].frame or 12

	    doorMap[mapDoor[i].x][mapDoor[i].y].id = i
		if level[mapDoor[i].x-1][mapDoor[i].y] > 0 and level[mapDoor[i].x+1][mapDoor[i].y] > 0 then
			doorMap[mapDoor[i].x][mapDoor[i].y].orientation = "h"
		elseif level[mapDoor[i].x][mapDoor[i].y-1] > 0 and level[mapDoor[i].x][mapDoor[i].y+1] > 0 then
			doorMap[mapDoor[i].x][mapDoor[i].y].orientation = "v"
		else
			doorMap[mapDoor[i].x][mapDoor[i].y].orientation = ""
		end
	end
end

function drawDoors()
    local stripIdx = 0
    for j=0,numRays-1 do
        setColor(0,255,255)

	    for i, v in pairs(visibleDoors) do
	        local tex = visibleDoors[i].texture or 13
            local rayScreenPos = (-numRays/2 + j)
            local rayViewDist = math.sqrt(rayScreenPos*rayScreenPos + viewDist*viewDist)
            local rayAngle = math.asin(rayScreenPos / rayViewDist)

            rayAngle = rayAngle % twoPI
            if (rayAngle < 0) then rayAngle = rayAngle + twoPI end

            local ix, iy = findIntersect(
                player.x, player.y, player.x + math.cos(player.rot + rayAngle)*10, player.y + math.sin(player.rot + rayAngle)*10,
                visibleDoors[i].lx1, visibleDoors[i].ly1, visibleDoors[i].lx2, visibleDoors[i].ly2, false, true)

            if ix ~= false then
                local dx = ix - player.x
                local dy = iy - player.y

                local dist = math.sqrt(dx*dx + dy*dy)

                local spriteAngle = math.atan2(dy, dx) - player.rot

                local size = viewDist / (math.cos(spriteAngle) * dist)

                local x = math.tan(spriteAngle) * viewDist
                local left = (screenWidth/2 + x)
                local top = ((screenHeight-size)*player.eyeHeight)

                local dbx = ix - player.x
                local dby = iy - player.y

                local blockDist = dbx*dbx + dby*dby
                local z = -math.floor((dist-.5)*10000)

                local sscale = size/tileSize
                if sscale < 0 then sscale = 0 end

                local lum = 1-(dist/fog_dist)
                if lum < 0 then lum = 0 end

                local textureX = 0

                if visibleDoors[i].orientation == "v" then
                    textureX = math.floor(((iy - visibleDoors[i].y + visibleDoors[i].slide) * tileSize) % tileSize)
                else
                    textureX = math.floor(((ix - visibleDoors[i].x + visibleDoors[i].slide) * tileSize) % tileSize)
                    lum = lum * .7
                end

                lum = lum * ambient_light

                local ql = wallQuad[tex][textureX]

                local dc = #drawCalls+1
                local rayScreenPos = stripIdx

                dist = math.sqrt(dist)
                dist = dist * math.cos(player.rot - rayAngle)

                if rayScreenPos > 0 then
                    if math.floor(ix) == math.floor(visibleDoors[i].x-1) and math.floor(iy) == math.floor(visibleDoors[i].y-1) then
                        drawCalls[dc] = {
                            z = z, x = rayScreenPos, y = top, id = math.random(1000,9999), dist = dist,
                            img = wallImg, quad = ql, sx = 2, sy = sscale, type = "door", lum = lum, tileid = 13,
                            mx = ix, my = iy
                        }
                    end
                end
            end
        end
		stripIdx = stripIdx + 1
    end
end

function processDoors()
	for i, v in pairs(visibleDoors) do
		local d = mapDoor[i]
		local dx1, dy1, dx2, dy2
        if visibleDoors[i].orientation == "v" then
            dx1 = d.x-.5
            dy1 = d.y-(d.slide-1)
            dx2 = d.x-.5
            dy2 = d.y-d.slide
        else
            dx1 = d.x-(d.slide-1)
            dy1 = d.y-.5
            dx2 = d.x-d.slide
            dy2 = d.y-.5
        end

        visibleDoors[i].frame = d.frame
        visibleDoors[i].texture = d.texture
        visibleDoors[i].slide = d.slide
        visibleDoors[i].x = d.x
        visibleDoors[i].y = d.y
        visibleDoors[i].lx1 = dx1
        visibleDoors[i].ly1 = dy1
        visibleDoors[i].lx2 = dx2
        visibleDoors[i].ly2 = dy2
    end
end

-- Checks if two lines intersect (or line segments if seg is true)
-- Lines are given as four numbers (two coordinates)
function findIntersect(l1p1x,l1p1y, l1p2x,l1p2y, l2p1x,l2p1y, l2p2x,l2p2y, seg1, seg2)
    local a1,b1,a2,b2 = l1p2y-l1p1y, l1p1x-l1p2x, l2p2y-l2p1y, l2p1x-l2p2x
    local c1,c2 = a1*l1p1x+b1*l1p1y, a2*l2p1x+b2*l2p1y
    local det,x,y = a1*b2 - a2*b1
    if det==0 then return false, "The lines are parallel." end
    x,y = (b2*c1-b1*c2)/det, (a1*c2-a2*c1)/det
    if seg1 or seg2 then
        local min,max = math.min, math.max
        if seg1 and not (min(l1p1x,l1p2x) <= x and x <= max(l1p1x,l1p2x) and min(l1p1y,l1p2y) <= y and y <= max(l1p1y,l1p2y)) or
           seg2 and not (min(l2p1x,l2p2x) <= x and x <= max(l2p1x,l2p2x) and min(l2p1y,l2p2y) <= y and y <= max(l2p1y,l2p2y)) then
            return false, "The lines don't intersect."
        end
    end
    return x,y
end

function checkForDoor(dist)
    dist = dist or 1
    local x1 = player.x
    local y1 = player.y
    local x2 = math.floor(x1 + math.cos(player.rot) * dist)+1
    local y2 = math.floor(y1 + math.sin(player.rot) * dist)+1
    if x2 > mapWidth then x2 = mapWidth end
    if y2 > mapHeight then y2 = mapHeight end
    if x2 < 1 then x2 = 1 end
    if y2 < 1 then y2 = 1 end

    return doorMap[x2][y2].id
end
