enemy = {}

function enemy:create(x, y, v)
    local instance = {}
    setmetatable(instance, {__index = self})
    instance:reset(x, y, v)
    return instance
end
function enemy:reset(x, y, v)
    v = v or {}
    enemyCount = enemyCount + 1
    self.id = "enemy_" .. enemyCount
    self.num = #actor+1
    self.x = x
    self.y = y
    self.alive = true
    self.rot = v.rot or 0
    self.rad = v.rad or .45
    self.mode = "idle"
    self.state = "stand"
    self.frame = 0
    self.speed = math.random(1,5)
    self.turnSpeed = 10
    self.homing = false
    self.alert = false
    self.hitWall = false
    self.xHit = 0
    self.yHit = 0
    self.pursueTimer = 0
    self.viewDistance = 20
    self.viewRayRes = .25
    self.target = {
        set = false,
        x = -1,
        y = -1,
        hostile = false,
        timer = 0
    }
    self.player = {
        canSee = false,
        angle = 0,
        dist = 0,
        ang1 = 0,
        ang2 = 0,
        pursueCountdown = 0
    }
    self.stuck = {
        inWall = false,
        nearest = 0
    }
    self.tile = {
        x = -1,
        y = -1
    }
end

function enemy:update(dt)
    self.tile.x = math.floor(self.x)
    self.tile.y = math.floor(self.y)
    actorAI(self, dt)
end

function enemy:draw()
    local x, y = self.x, self.y

    if debugMode == true and self.num == inspectActor then
        if self.player.canSee then
            love.graphics.setColor(255,255,0,50)
        else
            love.graphics.setColor(0,255,255,50)
        end
        love.graphics.triangle("fill",
            x * tileSize, y * tileSize,
            (x + math.cos(self.player.ang1) * 100) * tileSize, (y + math.sin(self.player.ang1) * 100) * tileSize,
            (x + math.cos(self.player.ang2) * 100) * tileSize, (y + math.sin(self.player.ang2) * 100) * tileSize
        )

        love.graphics.setColor(255,100,0)
        for i = 1, #self.chain do
            love.graphics.circle("fill", self.chain[i].x * tileSize, self.chain[i].y * tileSize, 2)
        end

        love.graphics.setColor(255,0,255)
        love.graphics.line(x * tileSize, y * tileSize, (x + math.cos(self.player.angle) * self.player.dist) * tileSize, (y + math.sin(self.player.angle) * self.player.dist) * tileSize)
    end

    if self.target.set then
        local s = 0.06 * tileSize
        if self.num == inspectActor and debugMode == true then
            s = 0.1 * tileSize
            love.graphics.setColor(0,255,0)
        else
            love.graphics.setColor(0,100,0)
        end
        love.graphics.circle("fill", self.target.x * tileSize, self.target.y * tileSize, s)
    end

    if self.num == inspectActor and debugMode == true then
        love.graphics.setColor(255,100,0)
    else
        if self.mode == "idle" then
            love.graphics.setColor(255,0,0)
        elseif self.mode == "search" then
            love.graphics.setColor(255,255,0)
        elseif self.mode == "pursue" then
            love.graphics.setColor(0,255,0)
        end
    end
    love.graphics.circle("line", x * tileSize, y * tileSize, self.rad * tileSize, 30)
    love.graphics.line(x * tileSize, y * tileSize, (x + math.cos(self.rot) * self.rad) * tileSize, (y + math.sin(self.rot) * self.rad) * tileSize)
end

function enemy:setTarget(x, y)
    if x or y then
        self.target.x = x
        self.target.y = y
    else
        self.target.x = math.random(2, map.w-1) + .5
        self.target.y = math.random(2, map.h-1) + .5
    end
    self.target.set = true
    self.hitWall = false
    self.target.timer = 10
end

function enemy:reachTarget()
    self.target.set = false
    self.target.hostile = false
    self.target.timer = 0
end

function enemy:move(dist, ang)
    local nx, ny
    local hitWall = false
    local x = self.x + math.cos(ang) * dist
	if x > self.x then
        local tr = isBlocking(x + self.rad, self.y - self.rad, self)
    	local br = isBlocking(x + self.rad, self.y + self.rad, self)
		if tr == false and br == false then
		    nx = x
		else
			nx = self.x
		    hitWall = true
		end
	elseif x <= self.x then
        local tl = isBlocking(x - self.rad, self.y - self.rad, self)
        local bl = isBlocking(x - self.rad, self.y + self.rad, self)
		if tl == false and bl == false then
			nx = x
		else
		    nx = self.x
		    hitWall = true
		end
	end

    local y = self.y + math.sin(ang) * dist
	if y > self.y then
        local bl = isBlocking(self.x - self.rad, y + self.rad, self)
        local br = isBlocking(self.x + self.rad, y + self.rad, self)
		if bl == false and br == false then
			ny = y
		else
		    ny = self.y
		    hitWall = true
		end
	elseif y <= self.y then
        local tl = isBlocking(self.x - self.rad, y - self.rad, self)
        local tr = isBlocking(self.x + self.rad, y - self.rad, self)
		if tl == false and tr == false then
			ny = y
		else
		    ny = self.y
		    hitWall = true
		end
	end

    for i, e in pairs(actor) do
        if self.id ~= e.id then
            local dist = distanceFrom(e.x, e.y, nx, ny)
            local combinedRad = self.rad + e.rad
            if dist < combinedRad then
                local ang = (findAngle(e.x, e.y, nx, ny))
                nx = e.x + math.sin(ang + (math.pi / 2)) * combinedRad
                ny = e.y + math.cos(ang + (math.pi / 2)) * combinedRad
            end
        end
    end

	return nx, ny, hitWall
end
