local math = math
local love = love
local pi = math.pi

function init()
	furthestWall = 0

	deg90 = math.rad(90)

	pixelScale = 1
	showMap = false
	miniMapZoom = 2
	miniMapScale = 8
	tileSize = 64
	setupVariables()

    mapItems = {}

	enemyType = {
		person_1 = { img = love.graphics.newImage("enemies/guard.png"), quad = {}, rad = .45 },
		person_2 = { img = love.graphics.newImage("enemies/guard.png"), quad = {}, rad = .45 },
		person_3 = { img = love.graphics.newImage("enemies/guard.png"), quad = {}, rad = .45 }
	}
	for i, e in pairs(enemyType) do
		e.img:setFilter("nearest", "nearest")
		for f = 0, 4 do
		    e.quad[f] = {}
    		for a = 0, 7 do
    			e.quad[f][a] = love.graphics.newQuad(a, f * tileSize, tileSize, tileSize, e.img:getWidth(), e.img:getHeight())
			end
		end
	end

	spriteObj = {
		img = love.graphics.newImage("images/sprites.png"),
		imgTall = love.graphics.newImage("images/sprites_tall.png"),
		quad = {},
		quadTall = {}
	}
	spriteObj.img:setFilter("nearest", "nearest")
	spriteObj.imgTall:setFilter("nearest", "nearest")
	local i = 0
	for x = 0, spriteObj.img:getWidth() / tileSize - 1 do
		for y = 0, spriteObj.img:getHeight() / tileSize - 1 do
			spriteObj.quad[i+1] = love.graphics.newQuad(i * tileSize, 0, tileSize, tileSize, spriteObj.img:getWidth(), spriteObj.img:getHeight())
			i = i + 1
		end
	end

	itemType = {
		table = { id = 1, solid = true, illum = false, rad = .45 },
		armor = { id = 2, solid = true, illum = false, rad = .4 },
		plant = { id = 3, solid = true, illum = false, rad = .35 },
		plant2 = { id = 4, solid = true, illum = false, rad = .35 },
		pillar = { id = 5, solid = true, illum = false, rad = .3 },
		lamp = { id = 6, solid = false, illum = true, rad = 0 },
		barrel = { id = 7, solid = true, illum = false, rad = .35 },
		pillartall = { id = 1, solid = true, illum = false, tall = true, rad = .3 },
		lamphang = { id = 2, solid = false, illum = true, tall = true, rad = 0 }
	}

	sky = {
		img = love.graphics.newImage("images/sky1.png"),
		quad = {},
		ox = 0,
		oy = 0
	}
	sky.w = sky.img:getWidth()
	sky.h = sky.img:getHeight()
	local qw = (sky.img:getWidth() / 4)
	for x = 0, 4 do
		sky.quad[x] = love.graphics.newQuad(x * qw, 0, qw, sky.img:getHeight(), sky.img:getWidth(), sky.img:getHeight())
	end

	mapItems = {}
	mapEnemy = {}
	mapDoor = {}

	love.filesystem.load("map1.lua")()
	mapWidth = #level
	mapHeight = #level[1]

	miniMap = {
		width = mapWidth * miniMapScale,	-- resize the internal canvas dimensions
		height = mapHeight * miniMapScale
	}

	--We keep the old map[] array around for legacy support as the raycasting engine still relies on it.
	--So we convert the level grid tables into something the caster can use.
	reticulateSplines()

	spriteMap = {}
	doorMap = {}
	for x = 1, mapWidth do
		spriteMap[x-1] = {}
		doorMap[x] = {}
		for y = 1, mapHeight do
			spriteMap[x-1][y-1] = -1
			doorMap[x][y] = { kind = "", state = 0, orientation = "v", id = 0 }
		end
	end
    loadDoors()

	sprite = {}
	for a, i in pairs(mapItems) do
		local h = 1
		if itemType[i.type].tall then h = 2 end
		sprite[a] = {
			x = i.x,
			y = i.y,
			img = itemType[i.type].id,
			solid = itemType[i.type].solid,
			visible = false,
			illum = itemType[i.type].illum,
			flicker = i.flicker,
			tall = h
		}
		spriteMap[math.floor(i.x)][math.floor(i.y)] = a
	end

	for d = 1, #mapDoor do
		floor[mapDoor[d].x][mapDoor[d].y] = 6
		doorMap[mapDoor[d].x][mapDoor[d].y].kind = mapDoor[d].kind
	end

	player = {
		x = 13.5,
		y = 6.5,
		dir = 0,
		rad = .45,
		rot = pi,
		speed = 0,
		moveSpeed = 5,
		accel = 0,
		strafe_accel = 0,
		rotSpeed = 180,
		strafe = 0,
		eyeHeight = 0.35,
		eyeStore = 0,
		bounce = 0,
		id = "player",
		keys = {
			left = false,
			right = false,
			forward = false,
			back = false,
			turn_left = false,
			turn_right = false
		}
	}

	setupMenu()

	wallImg = love.graphics.newImage("images/walls.png")
	wallImg:setFilter("nearest", "nearest")
	canvasWall = love.graphics.newCanvas(wallImg:getWidth(), wallImg:getHeight())
	canvasWall:setFilter("nearest", "nearest")
	love.graphics.setCanvas(canvasWall)
    love.graphics.draw(wallImg, 0, 0)
    love.graphics.setCanvas()

	local t = 0
	wallQuad = {}
	editorWall = {}
	for x = 0, wallImg:getWidth() / tileSize - 1 do
		for y = 0, wallImg:getHeight() / tileSize - 1 do
			t = t + 1
			wallQuad[t] = {}
			editorWall[t] = love.graphics.newQuad( (x * tileSize), y * tileSize, tileSize, tileSize, wallImg:getWidth(), wallImg:getHeight() )
			for s = 0, tileSize - 1 do
				wallQuad[t][s] = love.graphics.newQuad( (x * tileSize) + s, y * tileSize, 1, tileSize, wallImg:getWidth(), wallImg:getHeight() )
			end
		end
	end

	floorImg = love.graphics.newImage("images/floor.png")
	floorQuad = {}
	local t = 0
	for x = 0, floorImg:getWidth() / tileSize - 1 do
		for y = 0, floorImg:getHeight() / tileSize - 1 do
			t = t + 1
			floorQuad[t] = love.graphics.newQuad( x * (tileSize), y * tileSize, tileSize, tileSize, floorImg:getWidth(), floorImg:getHeight() )
		end
	end

	createCanvases()

	setupMiniMap()

	game = { mode = "game" }

	groundInner:clear(255,255,255,0)
	groundLight:clear(255,255,255,0)

    drawGround()

	groundQuad = {}
	for i = 0, ground:getHeight()-1 do
		groundQuad[i] = love.graphics.newQuad(0, ground:getHeight()-i, ground:getWidth(), 1, ground:getWidth(), ground:getHeight())
	end

    loadPixelEffects()
end

function reticulateSplines()
	map = {}
	map_upper = {}
	local i = 0
	for y = 1, mapHeight do
		for x = 1, mapWidth do
			i = i + 1
			map[i] = level[x][y]
			map_upper[i] = level_upper[x][y]
		end
	end
end

function castRays()
	local t = love.timer.getTime()
	furthestWall = 0
	visibleSprites = {}
	visibleDoors = {}

	local stripIdx = 0
	local foundTrans = false
	local transList = {}

	for i=0,numRays-1 do
		-- where on the screen does ray go through?
		local rayScreenPos = (-numRays/2 + i)
		-- the distance from the viewer to the point on the screen, simply Pythagoras.
		local rayViewDist = math.sqrt(rayScreenPos*rayScreenPos + viewDist*viewDist)
		-- the angle of the ray, relative to the viewing direction.
		-- right triangle: a = sin(A) * c
		local rayAngle = math.asin(rayScreenPos / rayViewDist)
	 	-- add the players viewing direction to get the angle in world space
		castSingleRay( player.rot + rayAngle, stripIdx)
		stripIdx = stripIdx + 1
	end
	debugger.time.castRays = love.timer.getTime() - t
end

function castSingleRay(rayAngle, stripIdx)
	-- first make sure the angle is between 0 and 360 degrees
	rayAngle = rayAngle % twoPI
--	if (rayAngle < 0) then rayAngle = rayAngle + twoPI end

	-- moving right/left? up/down? Determined by which quadrant the angle is in.
	local right = (rayAngle > twoPI * 0.75) or (rayAngle < twoPI * 0.25)
	local up = (rayAngle < 0) or (rayAngle > pi)

	-- only do these once
	local angleSin = math.sin(rayAngle)
	local angleCos = math.cos(rayAngle)

	local vHit = false
	local dist = 0	-- the distance to the block we hit
	local xHit = 0 	-- the x and y coord of where the ray hit the block
	local yHit = 0

	local textureX	-- the x-coord on the texture of the block, ie. what part of the texture are we going to render
	local wallX	-- the (x,y) map coords of the block
	local wallY

	-- first check against the vertical map/wall lines
	-- we do this by moving to the right or left edge of the block we're standing in
	-- and then moving in 1 map unit steps horizontally. The amount we have to move vertically
	-- is determined by the slope of the ray, which is simply defined as sin(angle) / cos(angle).
	local tileNum = nil
	local tileNum_u = nil
	local hitDoor = false

	local slope = angleSin / angleCos 	-- the slope of the straight line made by the ray

	-- we move either 1 map unit to the left or right
	local dX
	if right then dX = 1 else dX=-1 end

	local dY = dX * slope 		-- how much to move up or down

	-- starting horizontal position, at one of the edges of the current map block
	local x
	if right then x = math.ceil(player.x) else x = math.floor(player.x) end
	-- starting vertical position. We add the small horizontal step we just made, multiplied by the slope.
	local y = player.y + (x - player.x) * slope

	while (x >= 0 and x < mapWidth and y >= 0 and y < mapHeight) do
		local wallX
		if right then wallX = math.floor(x) else wallX = math.floor(x -1) end
		local wallY = math.floor(y)
		local inside = wallX > 0 and wallX <= mapWidth and wallY > 0 and wallY <= mapHeight

        if inside then
            if doorMap[wallX+1][wallY+1].kind ~= "" and doorMap[wallX+1][wallY+1].orientation == "v" then
                visibleDoors[doorMap[wallX+1][wallY+1].id] = { active = true, orientation = "v" }
            end
        end

        -- is this point inside a wall block?
        local k = 1+(math.floor(wallY))*mapWidth+math.floor(wallX)
        if map[k] > 0 then
            tileNum = map[k]
            tileNum_u = map_upper[k]

            local distX = x - player.x
            local distY = y - player.y
            dist = distX*distX + distY*distY	-- the distance from the player to this point, squared.

            -- where exactly are we on the wall? textureX is the x coordinate on the texture that we'll use when texturing the wall.
            textureX = y % 1

            -- if we're looking to the left side of the map, the texture should be reversed
            if (not right) then textureX = 1 - textureX end

            xHit = math.floor(x)
            yHit = math.floor(y)
            ix = x
            iy = y
            vHit = true

            if right then
                if doorMap[wallX][wallY+1].kind ~= "" and (xHit == wallX and yHit == wallY) and doorMap[wallX][wallY+1].orientation == "h" then
                    tileNum = 12
                end
            elseif not right then
                if doorMap[wallX+2][wallY+1].kind ~= "" and (xHit == wallX+1 and yHit == wallY) and doorMap[wallX+2][wallY+1].orientation == "h" then
                    tileNum = 12
                end
            end

            break
        end

        if inside then
            if spriteMap[wallX][wallY] > -1 then visibleSprites[spriteMap[wallX][wallY]] = true end
        end

        x = x+dX
        y = y+dY
	end

	-- now check against horizontal lines. It's basically the same, just "turned around".
	-- the only difference here is that once we hit a map block,
	-- we check if there we also found one in the earlier, vertical run. We'll know that if dist != 0.
	-- If so, we only register this hit if this distance is smaller.
	local slope = angleCos / angleSin
	local dY = up and -1 or 1

	local dX = dY * slope

	local y

	if up then y = math.floor(player.y) else y = math.ceil(player.y) end

	local x = player.x + (y - player.y) * slope
	while (x >= 0 and x < mapWidth and y >= 0 and y < mapHeight) do
		local wallY
		if up then wallY = math.floor(y - 1) else wallY = math.floor(y) end
		local wallX = math.floor(x)
		local inside = wallX > 0 and wallX <= mapWidth and wallY > 0 and wallY <= mapHeight

        if inside then
            if doorMap[wallX+1][wallY+1].kind ~= "" and doorMap[wallX+1][wallY+1].orientation == "h" then
                visibleDoors[doorMap[wallX+1][wallY+1].id] = { active = true, orientation = "h" }
            end
        end

		local k=1+(math.floor(wallY))*mapWidth+math.floor(wallX)
		if map[k] > 0 then
			local distX = x - player.x
			local distY = y - player.y
			local blockDist = distX*distX + distY*distY

			if (dist==0 or blockDist < dist) then
				dist = blockDist
				tileNum = map[k]
				tileNum_u = map_upper[k]

				xHit = math.floor(x)
				yHit = math.floor(y)
				ix = x
				iy = y
				vHit = false

				textureX = x % 1

				if (not up) then textureX = 1 - textureX end
			end

			if up then
				if doorMap[wallX+1][wallY+2].kind ~= "" and (xHit == wallX and yHit == wallY+1) and doorMap[wallX][wallY+1].orientation == "v" then
					tileNum = 12
				end
			elseif not up then
				if doorMap[wallX+1][wallY].kind ~= "" and (xHit == wallX and yHit == wallY) and doorMap[wallX][wallY+1].orientation == "v" then
					tileNum = 12
				end
			end

			break
		end

        if inside then
			if spriteMap[wallX][wallY] > -1 then visibleSprites[spriteMap[wallX][wallY]] = true end
		end

		x = x+dX
		y = y+dY
	end

	if (dist>0) then
		dist = math.sqrt(dist)
		dist = dist * math.cos(player.rot - rayAngle)
		local height = (viewDist / dist)

        if dist <= 50 then
    		createStrip(stripIdx, height, vHit, textureX, dist, tileNum, tileNum_u, ix, iy)
    	end
	end
end

function createStrip(stripIdx, height, vHit, textureX, dist, tileNum, tileNum_u, ix, iy)
	tileNum = tileNum or 4
	tileNum_u = tileNum_u or 1
	if dist > 50 then dist = 50 end
	darkness = 200 / dist

	textureX = math.floor(textureX * tileSize)
	local rayScreenPos = stripIdx

	if vHit == true then c = 0 else c = 1 end

	local scale = ((screenHeight+height)*0.5 - (screenHeight-height)*0.5) / tileSize

	z = -math.floor(dist*(10000))

	local qu, ql = nil, nil
	if tileNum > 0 then
		ql = wallQuad[tileNum][textureX]
	end
	if tileNum_u > 0 then
		qu = wallQuad[tileNum_u][textureX]
	end

	local lum = 1-(dist/fog_dist)
	if lum < 0 then lum = 0 end
	if vHit == false then lum = lum * .7 end
	lum = lum * ambient_light

    local dc = #drawCalls + 1
	drawCalls[dc] = {
		z = z, x = rayScreenPos, y = (screenHeight-height) * player.eyeHeight, id = math.random(1000,9999), dist = dist,
		img = canvasWall, quad = ql, quad_upper = qu, sx = 1, sy = scale, type = "wall", lum = lum, tileid = tileNum,
        mx = ix, my = iy
	}

	if furthestWall < dist then
		furthestWall = dist
		furthestStrip = stripIdx
		furthestTop = math.floor((screenHeight-height) * player.eyeHeight)
		furthestBottom = math.floor(furthestTop + scale * tileSize)
	end
end

function sort(T) table.sort(T, function(a, b) return a.z < b.z end ) end

function setupVariables()
	windowWidth = 320
	windowHeight = 200
	screenWidth = windowWidth / pixelScale
	screenHeight = windowHeight / pixelScale

	fov = 60 * pi / 180
	numRays = math.ceil(screenWidth)
	fovHalf = fov / 2
	viewDist = (screenWidth/2) / math.tan((fov / 2))
	twoPI = pi * 2

	raycasts = {}

	local ss1 = love.graphics.getHeight() / windowHeight
	local ss2 = love.graphics.getWidth() / windowWidth
	if ss1 < ss2 then screenScale = ss1 else screenScale = ss2 end

	screenOffsetX = (love.graphics.getWidth() - (windowWidth * screenScale)) / 2
	screenOffsetY = (love.graphics.getHeight() - (windowHeight * screenScale)) / 2

end

function createCanvases()
	mapCanvas = love.graphics.newCanvas(miniMap.width, miniMap.height)
	mapCanvas:setFilter("nearest","nearest")

	overlay = love.graphics.newCanvas(windowWidth, windowHeight)
	overlay:setFilter("nearest", "nearest")

	status = love.graphics.newCanvas(windowWidth/2, windowHeight/2)
	status:setFilter("nearest", "nearest")

	canvas = love.graphics.newCanvas(screenWidth, screenHeight)
	canvas:setFilter("nearest", "nearest")

	background = love.graphics.newCanvas(screenWidth, screenHeight)
	background:setFilter("nearest", "nearest")

	ground = love.graphics.newCanvas(2048, 2048)
	ground:setFilter("nearest", "nearest")
	groundInner = love.graphics.newCanvas(mapWidth * tileSize, mapHeight * tileSize)
	groundInner:setFilter("nearest", "nearest")
	groundLight = love.graphics.newCanvas(mapWidth * tileSize, mapHeight * tileSize)
	groundLight:setFilter("nearest", "nearest")

	setCanvas()
end

function drawStatusBar()
	setCanvas(status)
	status:clear(0,0,0,0)

--	setColor(255,255,255)
--	grPrint("Press ESCAPE to open the MENU", 2, windowHeight / 2 - 10)
end

function getScreenRes()
	local sm = -1
	for i, m in pairs(menu.screenmodes) do
		if love.graphics.getWidth() == m.width and love.graphics.getHeight() == m.height then
			sm = i
		end
	end
	return sm
end

function ai(i, dt)
    local e = mapEnemy[i]
    e.target.x = player.x
    e.target.y = player.y
    local dx = e.target.x - e.x
    local dy = e.target.y - e.y
    local dist = math.sqrt(dx*dx + dy*dy)
    if e.prevDist == nil then e.prevDist = dist end
    if dist > 2 then
        local angle = -findAngle(player.x, player.y, e.x, e.y) - math.pi
        --local angle = math.atan2(dy, dx)
        e.rotDeg = angle * 180 / pi
        e.rot = angle
        e.speed = 1.5
        if dist > 3 then e.speed = 2 end
        local walkCycleTime = 1000
        local numWalkSprites = 4
        e.frame = (e.frame + dt * (numWalkSprites * (e.speed * .6))) % numWalkSprites
        e.state = "walk"
    else
        e.state = "stand"
        e.speed = 0
    end

    if e.prevDist == math.floor(dist) then
        e.distCount = e.distCount + dt
        if e.distCount > 1 then
            e.target.x = e.x
            e.target.y = e.y
        end
    end

    e.prevDist = math.floor(dist)

    e.x, e.y = move(e, e.speed * dt, e.rot)
end

function isBlocking(x,y,self)
    local block = false
	x = math.floor(x)
	y = math.floor(y)
	if x < 0 then x = 0 end
	if y < 0 then y = 0 end

	if doorMap[x+1][y+1].id > 0 then
	    if mapDoor[doorMap[x+1][y+1].id].opened == false then
    		block = true
    	end
	elseif spriteMap[x][y] > -1 then
		block = sprite[spriteMap[x][y]].solid
	elseif (y < 0 or y > mapHeight or x < 0 or x > mapWidth) then
		block = true
	else
		block = level[x+1][y+1] > 0
	end

	return block
end

function move(self, dist, ang)
    local nx, ny
    local blocked = false
    local x = self.x + math.cos(ang) * dist
	if x > self.x then
        local tr = isBlocking(x + self.rad, self.y - self.rad, self)
    	local br = isBlocking(x + self.rad, self.y + self.rad, self)
		if tr == false and br == false then
		    nx = x
		else
			nx = self.x
            blocked = true
		end
	elseif x <= self.x then
        local tl = isBlocking(x - self.rad, self.y - self.rad, self)
        local bl = isBlocking(x - self.rad, self.y + self.rad, self)
		if tl == false and bl == false then
			nx = x
		else
		    nx = self.x
            blocked = true
		end
	end

    local y = self.y + math.sin(ang) * dist
	if y > self.y then
        local bl = isBlocking(self.x - self.rad, y + self.rad, self)
        local br = isBlocking(self.x + self.rad, y + self.rad, self)
		if bl == false and br == false then
			ny = y
		else
		    ny = self.y
            blocked = true
		end
	elseif y <= self.y then
        local tl = isBlocking(self.x - self.rad, y - self.rad, self)
        local tr = isBlocking(self.x + self.rad, y - self.rad, self)
		if tl == false and tr == false then
			ny = y
		else
		    ny = self.y
            blocked = true
		end
	end

    for i, e in pairs(mapEnemy) do
        if self.id ~= e.id then
            local dist = distanceFrom(e.x, e.y, nx, ny)
            local combinedRad = self.rad + e.rad
            if dist < combinedRad then
                local ang = (findAngle(e.x, e.y, nx, ny))
                nx = e.x + math.sin(ang + (math.pi / 2)) * combinedRad
                ny = e.y + math.cos(ang + (math.pi / 2)) * combinedRad
                blocked = true
            end
        end
    end

	return nx, ny, blocked
end

function updatePlayer(dt)
	if player.keys.forward then
		player.eyeStore = player.eyeStore + dt
 		player.speed = 1
		player.accel = player.accel + dt * 5
		if player.accel > 1 then player.accel = 1 end
	elseif player.keys.back then
		player.eyeStore = player.eyeStore - dt
		player.speed = -1
		player.accel = player.accel + dt * 5
		if player.accel > 1 then player.accel = 1 end
	else
		player.accel = 0
		player.speed = 0
	end

	player.eyeHeight = 0.35 + math.cos(player.eyeStore * 10) / 100

	if player.keys.left then
		player.strafe = -1
		player.strafe_accel = player.strafe_accel + dt * 5
		if player.strafe_accel > 1 then player.strafe_accel = 1 end
	elseif player.keys.right then
		player.strafe = 1
		player.strafe_accel = player.strafe_accel + dt * 5
		if player.strafe_accel > 1 then player.strafe_accel = 1 end
	else
		player.strafe = 0
		player.strafe_accel = 0
	end

	if player.keys.turn_left then
		player.dir = -1
	elseif player.keys.turn_right then
		player.dir = 1
	else
		player.dir = 0
	end


	local moveStep = player.speed * player.moveSpeed * player.accel * dt
	-- player will move this far along the current direction vector
	player.rot = (player.rot + player.dir * math.rad(player.rotSpeed) * dt) % twoPI
	-- add rotation if player is rotating (player.dir != 0)

	if moveStep ~= 0 then
        player.x, player.y = move(player, moveStep, player.rot)
	end

	if player.strafe ~= 0 then
        player.x, player.y = move(player, player.moveSpeed * player.strafe_accel * dt, player.rot + math.rad(player.strafe * 90))
	end
end

function distanceFrom(x1,y1,x2,y2) return math.sqrt((x2 - x1) ^ 2 + (y2 - y1) ^ 2) end

function findAngle(x1, y1, x2, y2)
	local t = (math.atan2(y1 - y2, x2 - x1)) % twoPI
	return t
end

function loadPixelEffects()
    effect = {}
    effect.blur_vertical = love.graphics.newPixelEffect[[
        extern number rt_h = 600.0; // render target height

        extern number intensity = 1.0;

        const number offset[3] = number[](0.0, 1.3846153846, 3.2307692308);
        const number weight[3] = number[](0.2270270270, 0.3162162162, 0.0702702703);

        vec4 effect(vec4 color, Image texture, vec2 texture_coords, vec2 pixel_coords)
        {
            vec4 texcolor = Texel(texture, texture_coords);
            vec3 tc = texcolor.rgb * weight[0];

            tc += Texel(texture, texture_coords + intensity * vec2(0.0, offset[1])/rt_h).rgb * weight[1];
            tc += Texel(texture, texture_coords - intensity * vec2(0.0, offset[1])/rt_h).rgb * weight[1];

            tc += Texel(texture, texture_coords + intensity * vec2(0.0, offset[2])/rt_h).rgb * weight[2];
            tc += Texel(texture, texture_coords - intensity * vec2(0.0, offset[2])/rt_h).rgb * weight[2];

            return color * vec4(tc, texcolor.a);
        }
    ]]
    effect.blur_vertical:send("rt_h", screenWidth)

    effect.blur_horizontal = love.graphics.newPixelEffect[[
        extern number rt_w = 800.0; // render target width

        extern number intensity = 1.0;

        const number offset[3] = number[](0.0, 1.3846153846, 3.2307692308);
        const number weight[3] = number[](0.2270270270, 0.3162162162, 0.0702702703);

        vec4 effect(vec4 color, Image texture, vec2 texture_coords, vec2 pixel_coords)
        {
            vec4 texcolor = Texel(texture, texture_coords);
            vec3 tc = texcolor.rgb * weight[0];

            tc += Texel(texture, texture_coords + intensity * vec2(offset[1], 0.0)/rt_w).rgb * weight[1];
            tc += Texel(texture, texture_coords - intensity * vec2(offset[1], 0.0)/rt_w).rgb * weight[1];

            tc += Texel(texture, texture_coords + intensity * vec2(offset[2], 0.0)/rt_w).rgb * weight[2];
            tc += Texel(texture, texture_coords - intensity * vec2(offset[2], 0.0)/rt_w).rgb * weight[2];

            return color * vec4(tc, texcolor.a);
        }
    ]]
    effect.blur_horizontal:send("rt_w", screenWidth)


    effect.wavy = love.graphics.newPixelEffect[[
        extern number time;
        extern number distortion;

        vec4 effect(vec4 color, Image tex, vec2 tc, vec2 pc)
        {
           // play with parameters here:
           tc.y += sin(tc.y * 10 + time * 10.0) * .03  * distortion;
           return Texel(tex,tc);
        }
    ]]

    useEffect = nil--effect.wavy
end
