math.randomseed(os.time()) math.random() math.random() math.random()

local math = math
local love = love

require 'shortlove'
require 'fonts'
require 'menu'
require 'world'
require 'game'
require 'minimap'
require 'doors'
require 'background'
require 'editor'
require 'testbed'
require 'enemy'

function love.load()
	debugger = {
		info = false,
		time = {
			simulate = 0,
			draw = 0,
			update = 0,
			castRays = 0,
			drawFloor = 0,
			delta = 0
		},
		topdelta = 0,
		maxfps = 0,
		count = {
			setCanvas = 0,
			setColor = 0,
			drawq = 0,
			draw = 0,
			print = 0
		}
	}

	editor = {
		zoom = .5,
		ground = true,
		level1 = true,
		level2 = true,
		enemies = true,
		things = true,
		other = true
	}

	love.graphics.setLineStyle("rough")

	init()
end

function love.update(dt)

--	local intensity = (math.sin(love.timer.getTime()) + 1) * 0.5
--	effect.blur_vertical:send("intensity", (intensity))
--	effect.blur_horizontal:send("intensity", (intensity))
--	effect.wavy:send("time", love.timer.getTime())
--	effect.wavy:send("distortion", 10)

	--[[love.graphics.setCanvas(canvasWall)
    canvasWall:clear(255,255,255,0)
    love.graphics.draw(wallImg, 0, 0)
    love.graphics.line(math.random(0,wallImg:getWidth()), math.random(0,wallImg:getHeight()), math.random(0,wallImg:getWidth()), math.random(0,wallImg:getHeight()))
    love.graphics.setCanvas()]]

	debugger.count.setCanvas = 0
	debugger.count.setColor = 0
	debugger.count.drawq = 0
	debugger.count.draw = 0
	debugger.count.print = 0

	fps = love.timer.getFPS()
	if dt > debugger.topdelta then debugger.topdelta = dt end
	if fps > debugger.maxfps then debugger.maxfps = fps end

	love.graphics.setCaption("A game in the third dimension (FPS: " .. fps .. ", Max FPS: " .. debugger.maxfps .. ", Top DT: " .. debugger.topdelta .. ")")

	debugger.time.delta = dt
	local ti = love.timer.getTime()

	if game.mode == "game" or game.mode == "testbed" then
		mx, my = math.floor(love.mouse.getX() / pixelScale / screenScale), math.floor(love.mouse.getY() / pixelScale / screenScale)
		if menu.opened == false then
			--ai(dt)

			player.keys.forward = love.keyboard.isDown("up")
			player.keys.back = love.keyboard.isDown("down")
			player.keys.left = love.keyboard.isDown("left")
			player.keys.right = love.keyboard.isDown("right")
			player.keys.turn_left = love.keyboard.isDown("z")
			player.keys.turn_right = love.keyboard.isDown("c")

			if love.keyboard.isDown("=") then
				miniMapZoom = miniMapZoom + dt
				if miniMapZoom > 10 then
					miniMapZoom = 10
				end
			elseif love.keyboard.isDown("-") then
				miniMapZoom = miniMapZoom - dt
				if miniMapZoom < .25 then
					miniMapZoom = .25
				end
			end

			updatePlayer(dt)

			simulate(dt)
		end
	elseif game.mode == "editor" then
		mx, my = math.floor(love.mouse.getX() / tileSize * (1 / editor.zoom)), math.floor(love.mouse.getY() / tileSize * (1 / editor.zoom))
	end
	debugger.time.update = love.timer.getTime() - ti
end

function love.draw()
	local ti = love.timer.getTime()
	if game.mode == "game" then
		if menu.opened == false then
			drawEverything()

			setCanvas()

			setColor(255,255,255)
			draw(overlay, screenOffsetX, screenOffsetY, 0, screenScale, screenScale)
		else
			setCanvas()
			displayMenu()
		end

		if debugger.info then
    		love.graphics.setFont(font["dialog"])
			local t = ""
			t = t .. "Pool #: " .. #drawPool .. "\n"
			t = t .. "Player:\n X: " .. player.x .. "\n Y: " .. player.y .. "\n "
			t = t .. "Rot: " .. player.rot .. "\n "
			t = t .. "Accelleration: " .. player.accel .. " - " .. player.strafe_accel .. "\n\n"

			t = t .. "Update:			 " .. debugger.time.update .. "\n"
			t = t .. " Simulate:	 " .. debugger.time.simulate .. "\n"
			t = t .. " Cast Rays:	" .. debugger.time.castRays .. "\n"
			t = t .. " Cast Floor: " .. debugger.time.drawFloor .. "\n"
			t = t .. " Other:			" .. debugger.time.update - (debugger.time.simulate + debugger.time.castRays + debugger.time.drawFloor) .. "\n"
			t = t .. "Draw:				 " .. debugger.time.draw .. "\n"
			t = t .. " World:			" .. debugger.time.world .. "\n"
			t = t .. " Other:			" .. debugger.time.draw - debugger.time.world .. "\n"

			t = t .. "Delta Time:	 " .. debugger.time.delta .. "\n\n"

			t = t .. "Mouse: " .. mx .. ", " .. my .. "\n"
			t = t .. "Cur Strip: " .. currentStrip .. "\n"
			t = t .. "Furthest:\n Wall:	" .. furthestWall .. "\n"
			t = t .. " Strip: " .. furthestStrip .. "\n"
			t = t .. " Top: " .. furthestTop .. "\n"
			t = t .. " Bottom: " .. furthestBottom .. "\n\n"

			t = t .. "setCanvas: " .. debugger.count.setCanvas .. "\n"
			t = t .. "setColor: " .. debugger.count.setColor .. "\n"
			t = t .. "drawq: " .. debugger.count.drawq .. "\n"
			t = t .. "draw: " .. debugger.count.drawq .. "\n"
			t = t .. "print: " .. debugger.count.print .. "\n"

			setColor(0,0,0)
			grPrint(t, 3, 3)
			setColor(255,255,255)
			grPrint(t, 2, 2)
		end

	elseif game.mode == "editor" then
        editorDraw()
	elseif game.mode == "testbed" then
        testbedDraw()
	end
	debugger.time.draw = love.timer.getTime() - ti
end

function love.keypressed(key)
	if game.mode == "game" or game.mode == "testbed" then
		if menu.opened == false then
			if key == "x" then
			    print("SHOOT!")

                local data = ""
                for i = 1, #drawCalls do
                    if drawCalls[i].type == "wall" or drawCalls[i].type == "door" then
                        data = data .. "\nCall #" .. i .. " (" .. drawCalls[i].type .. ")\n"
                        for j, c in pairs(drawCalls[i]) do
                            local k, v = tostring(j), tostring(c)
                            if k == "dist" or k == "z" or k == "mx" or k == "my" or k == "x" or k == "y" then
                                data = data .. k .. " = " .. v .. "\n"
                            end
                        end
                    end
                end

                love.filesystem.write("dump.txt", data, all)
			end
			if key == "s" or key == " " then
			    local d = 0
                for i = 0, 4 do
                    c = i / 2
                    if d == 0 then
                        d = checkForDoor(c)
                    end
                end
                if d > 0 then
                    mapDoor[d].opening = not mapDoor[d].opening
                end
			end

			if key == "tab" then showMap = not showMap end
			if key == "d" then debugger.info = not debugger.info end
			if key == "/" then
				menu.options[1].onRight(menu.options[1])
				setupVariables()
				createCanvases()
			end
			if key == "escape" then openMenu() end
			if key == "p" then
				setupMiniMap()
			end
			if key == "'" then
			    mapDoor[1].opening = not mapDoor[1].opening
			end
		else
			if key == "escape" then closeMenu() end
			if key == "up" then
				menu.selection = menu.selection - 1
				if menu.selection <= 0 then menu.selection = #menu.options end
			end
			if key == "down" then
				menu.selection = menu.selection + 1
				if menu.selection > #menu.options then menu.selection = 0 end
			end
			if key == "left" then menu.options[menu.selection].onLeft(menu.options[menu.selection]) end
			if key == "right" then menu.options[menu.selection].onRight(menu.options[menu.selection]) end

		end
		if key == "e" then
			game.mode = "editor"
		end
	elseif game.mode == "editor" then
		if key == "e" then
			game.mode = "game"
		elseif key == "-" then
			editor.zoom = editor.zoom - .25
		elseif key == "=" then
			editor.zoom = editor.zoom + .25
		elseif key == "1" then
			editor.ground = not editor.ground
		elseif key == "2" then
			editor.level1 = not editor.level1
		elseif key == "3" then
			editor.level2 = not editor.level2
		end
	elseif game.mode == "testbed" then

	end
	if key == "t" then
	    if game.mode == "testbed" then
	        game.mode = "game"
	    else
	        game.mode = "testbed"
        end
	end
end

function love.mousepressed(x, y, b)
	if game.mode == "game" then
		if love.keyboard.isDown("lalt") then
			if b == "wu" then
				player.eyeHeight = player.eyeHeight + 0.01
				if player.eyeHeight > 1 then
					player.eyeHeight = 1
				end
			elseif b == "wd" then
				player.eyeHeight = player.eyeHeight - 0.01
				if player.eyeHeight < 0 then
					player.eyeHeight = 0
				end
			end
		else
			local add = 0.05
			if love.keyboard.isDown("lshift") then add = 0.1 end
			if b == "wu" then
				ambient_light = ambient_light + add
				if ambient_light > 1 then
					ambient_light = 1
				end
			elseif b == "wd" then
				ambient_light = ambient_light - add
				if ambient_light < 0 then
					ambient_light = 0
				end
			end
		end
	elseif game.mode == "editor" then
		if b == "l" then
			level[mx+1][my+1] = 10
			reticulateSplines()
		elseif b == "r" then
			level[mx+1][my+1] = 0
			reticulateSplines()
		end
	end
end
