local math = math
local pi = math.pi

function simulate(dt)
	drawCalls = {}
	drawPool = {}
	extraLines = {}

	castRays()

    updateDoors(dt)
    processDoors()

	local t = love.timer.getTime()
	for i, v in pairs(visibleSprites) do
		local s = sprite[i]
		local dx = s.x - player.x
		local dy = s.y - player.y

		local dist = math.sqrt(dx*dx + dy*dy)

		local spriteAngle = math.atan2(dy, dx) - player.rot

		local size = viewDist / (math.cos(spriteAngle) * dist)

		local x = math.tan(spriteAngle) * viewDist
		local left = (screenWidth/2 + x)
		local top = ((screenHeight-size)*player.eyeHeight)

		local dbx = s.x - player.x
		local dby = s.y - player.y

		local blockDist = dbx*dbx + dby*dby
		local z = -math.floor((dist-.5)*10000)

		local lum = 1-(dist/fog_dist)
		if lum < 0 then lum = 0 end
		lum = lum * ambient_light
		if s.illum then lum = 1 end

        local dc = #drawCalls+1
		drawCalls[dc] = {
			z = z, x = left, y = top, id = math.random(1000,9999), dist = dist, flicker = s.flicker,
			img = s.img, sx = size/tileSize, sy = size/tileSize, type = "sprite", illum = s.illum, tall = s.tall, lum = lum
		}
	end

	for i, e in pairs(mapEnemy) do
        ai(i, dt)

		local en = mapEnemy[i]
		local dx = en.x - player.x
		local dy = en.y - player.y

		local dist = math.sqrt(dx*dx + dy*dy)

		local spriteAngle = math.atan2(dy, dx) - player.rot

		local size = viewDist / (math.cos(spriteAngle) * dist)

		local x = math.tan(spriteAngle) * viewDist
		local left = (screenWidth/2 + x)
		local top = ((screenHeight-size)*player.eyeHeight)

		local dbx = en.x - player.x
		local dby = en.y - player.y

		local blockDist = dbx*dbx + dby*dby;
		local z = -math.floor(dist*10000)-10

		local sscale = size/tileSize
		if sscale < 0 then sscale = 0 end

		local lum = 1-(dist/fog_dist)
		if lum < 0 then lum = 0 end
		lum = lum * ambient_light
		if en.illum then lum = 1 end

        local s
        if mapEnemy[i].state == "stand" then
            s = 0
        else
            s = 1 + math.floor(en.frame)
        end

        if sscale > 0 then
            local dc = #drawCalls+1
            drawCalls[dc] = {
                z = z, x = left, y = top, id = en.id, dist = dist,
                img = enemyType[en.type].img, quad = enemyType[en.type].quad[s][0],
                class = en.type, sx = sscale, sy = sscale, type = "enemy", lum = lum
            }
        end
	end

    if mx > 0 then
    	currentStrip = drawCalls[mx].dist
    end

	debugger.time.simulate = love.timer.getTime() - t
end

function drawWorld()
	local t = love.timer.getTime()

    drawDoors()

	sort(drawCalls)

	for j, c in pairs(drawCalls) do
		local i = #drawPool + 1
		drawPool[i] = c.id
	end

	for i = 0, #drawPool do
		local spr = drawCalls[i]
		if spr ~= nil then
			if spr.type == "wall" then

				if spr.quad ~= nil then
					setColor(255*spr.lum,255*spr.lum,255*spr.lum)
					drawq(spr.img, spr.quad, spr.x, spr.y, 0, 1, spr.sy)
				end
				if mapLevels > 1 and spr.quad_upper ~= nil then
					for i = 2, math.floor(mapLevels) do
						local y = spr.sy*tileSize*(i-1)
						--If the image is completely off screen, don't bother drawing it.
                        if spr.y-y+(spr.sy*tileSize) < 0 then
                            break
                        end
						setColor(255*spr.lum,255*spr.lum,255*spr.lum)
						drawq(spr.img, spr.quad_upper, spr.x, spr.y-y, 0, 1, spr.sy)
					end
				end

			elseif spr.type == "door" then

				if spr.quad ~= nil then
					setColor(255*spr.lum,255*spr.lum,255*spr.lum)
					drawq(spr.img, spr.quad, spr.x, spr.y, 0, 3, spr.sy)
				end

			elseif spr.type == "sprite" then

				local halfWidth = (spr.sx*tileSize)/2
				setColor(255*spr.lum,255*spr.lum,255*spr.lum)
				drawq(spriteObj.img, spriteObj.quad[spr.img], spr.x-halfWidth, spr.y + spr.sy * tileSize, 0, spr.sx, spr.sy, 0, tileSize * spr.tall)

			elseif spr.type == "enemy" then

				local halfWidth = (spr.sx*tileSize)/2
				setColor(255*spr.lum,255*spr.lum,255*spr.lum)
				drawq(spr.img, spr.quad, spr.x-halfWidth, spr.y + spr.sy * tileSize, 0, spr.sx, spr.sy, 0, tileSize * 1)

                love.graphics.setFont(font["tiny"])
                local t = ""
				for j, d in pairs(spr) do
                    t = t .. j .. " = " .. tostring(d) .. "\n"
				end
                if debugger.info then
                    love.graphics.setColor(0,0,0,100)
                    love.graphics.rectangle("fill", spr.x, 0, 90, 80)
                    love.graphics.setColor(255,255,255)
                    love.graphics.print(t, spr.x, 0)
                end
			end
		end
	end

	debugger.time.world = love.timer.getTime() - t
end

function drawEverything()
	processFloor()

	setCanvas(canvas)
	canvas:clear(255,255,255,0)
	setColor(255,255,255)

	drawSky()
	drawFloor()
    drawCeiling()

	drawWorld()

	drawStatusBar()

    love.graphics.setPixelEffect(useEffect)
	setCanvas(overlay)
	overlay:clear(0,0,0,0)

--	draw(background, 0, 0, 0, pixelScale, pixelScale)

	setColor(255,255,255)
	draw(canvas, 0, 0, 0, pixelScale, pixelScale)
    love.graphics.setPixelEffect()

	setColor(0,0,0,255*.5)
	draw(status, 2, 2, 0, 2, 2)
	setColor(255,255,255)
	draw(status, 0, 0, 0, 2, 2)

	if showMap then drawMiniMap() end
end
